import 'dart:ui';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
//import 'package:latlong/latlong.dart';
import 'constants.dart' as constants;
import 'package:tcp_socket_connection/tcp_socket_connection.dart';
import 'dart:convert';
import 'package:intl/intl.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'package:we_clean_project/meetingMarker.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';

class MeetingSettings extends StatefulWidget {
  MeetingSettings({Key key, this.position}) : super(key: key);
  final LatLng position;

  @override
  _MeetingSettings createState() => _MeetingSettings();
}

class _MeetingSettings extends State<MeetingSettings> {
  var enDatesFuture = initializeDateFormatting('en', null);
  DateUtility dateUtility;
  List<DateTime> allDates;
  List<DateTime> datesToShow;
  DateTime selectedDateTime;
  String selectedMonth;
  int selectedYear;
  int indexStartHour=24;
  int indexEndHour=24;
  TcpSocketConnection server=new TcpSocketConnection("192.168.1.110", constants.SERVER_PORT);
  LatLng position;
  var textDialogTitleController = TextEditingController();
  var textDialogDescriptionController = TextEditingController();
  var textDialogNumberController = TextEditingController();

  @override
  void initState() {
    dateUtility=new DateUtility();
    selectedDateTime=DateTime.now();
    selectedMonth=dateUtility.getMonthName(DateTime.now());
    selectedYear=selectedDateTime.year;
    allDates=dateUtility.getDaysInBeteween(DateTime.now(), dateUtility.getEndDate(DateTime.now()));
    populateCalendar(new DateTime(DateTime.now().year,DateTime.now().month,1), dateUtility.getLastDayInMonth(DateTime.now()));
    position=widget.position;
    super.initState();
  }



  void populateCalendar(DateTime startDate, DateTime endDate){
    setState(() {
      datesToShow=dateUtility.getDaysInBeteween(startDate, endDate);
    });
  }

  Color getDayHighlightColor(DateTime dateTime){
    //print(dateUtility.getDaysInMonth(dateTime));
    if(!dateUtility.isSameDate(selectedDateTime, dateTime)){
      return constants.LIGHT_GREEN;
    }
    return constants.DARK_GREEN;
  }

  double getHeight(){
    var padding = MediaQuery.of(context).padding;
    double height = MediaQuery.of(context).size.height;
    double newheight = height - padding.top - padding.bottom-225;
    return newheight;
  }
  double getMinBottomHeight(){
    double height=getHeight();
    double newheight=height-503;
    if(newheight>=0)return newheight;
    return 10;
  }

  Color getColorStartHour(int index){
    if(indexStartHour!=index){
      return constants.FADING_GREY;
    }
    return constants.GREY;
  }

  Color getColorEndHour(int index){
    if(indexEndHour!=index){
      return constants.FADING_GREY;
    }
    return constants.GREY;
  }

  TextStyle getMonthHighlightStyle(String month){
    if(month.compareTo(selectedMonth)!=0){
      return TextStyle(color: constants.FADING_WHITE, fontSize: 20);
    }
    return TextStyle(color: constants.WHITE, fontSize: 20,);
  }

  Color getMonthHighlightColor(String month){
    if(month.compareTo(selectedMonth)!=0){
      return constants.LIGHT_GREEN;
    }
    return constants.WHITE;
  }

  void onBackPressed(){
    Navigator.pop(context);
  }

  void onDayPress(DateTime dateTime){
    setState(() {
      selectedDateTime=dateTime;
    });
  }

  void addYearPress(){
    setState(() {
      selectedYear+=1;
    });
    int monthNumber=dateUtility.getMonthNumber(selectedMonth);
    DateTime startDate=new DateTime(selectedYear,monthNumber,1);
    DateTime endDate=dateUtility.getLastDayInMonth(startDate);
    populateCalendar(startDate, endDate);
  }

  void removeYearPress(){
    setState(() {
      selectedYear-=1;
    });
    int monthNumber=dateUtility.getMonthNumber(selectedMonth);
    DateTime startDate=new DateTime(selectedYear,monthNumber,1);
    DateTime endDate=dateUtility.getLastDayInMonth(startDate);
    populateCalendar(startDate, endDate);
  }

  void setStartIndex(int index){
    setState(() {
      indexStartHour=index;
    });
  }
  void setEndIndex(int index){
    setState(() {
      indexEndHour=index;
    });
  }

  void onMonthPress(String month){
    setState(() {
      selectedMonth=month;
    });
    int monthNumber=dateUtility.getMonthNumber(month);
    DateTime startDate=new DateTime(DateTime.now().year,monthNumber,1);
    DateTime endDate=dateUtility.getLastDayInMonth(startDate);
    populateCalendar(startDate, endDate);
  }

  void sendMeetingToServer()async {
    if(indexEndHour<=indexStartHour){
      print("ERRORE");
      return;
    }
    String description=textDialogDescriptionController.text;
    String title=textDialogTitleController.text;
    String number=textDialogNumberController.text;
    if(description==null || title ==null || number==null ||title.length<=5 || description.length<=5 || number.length<1){
      print("ERRORE");
      return;
    }
    int n=0;
    try {
      n=int.parse(number);
    }catch(Exception){
      print("ERRORE");
      return;
    }
    if(n<5||n>50){
      print("ERRORE");
      return;
    }

    MeetingMarker m=new MeetingMarker();
    m.maxPartecipants=n;
    m.actualPartecipants=0;
    m.startDateTime=selectedDateTime.year.toString()+"-"+selectedDateTime.month.toString()+"-"+selectedDateTime.day.toString()+" "+dateUtility.orari[indexStartHour];
    m.endDateTime=selectedDateTime.year.toString()+"-"+selectedDateTime.month.toString()+"-"+selectedDateTime.day.toString()+" "+dateUtility.orari[indexEndHour];
    m.title=title;
    m.description=description;
    m.latitude=position.latitude;
    m.longitude=position.longitude;
    print(m.toString());

    await server.connect(5000, "EOS", receiveCallback);
    //String asd=jsonEncode(m);
    //print(asd);
    server.sendMessage("ADDMEETING%_%"+jsonEncode(m).toString()+"%_%EOS");
  }

  void receiveCallback(String message){
    if(message.startsWith("MEETINGSAVED")){
      server.disconnect();
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return  Scaffold(
      backgroundColor: constants.LIGHT_GREEN,
      body: AnnotatedRegion<SystemUiOverlayStyle>(
        value: SystemUiOverlayStyle(
          statusBarColor: constants.LIGHT_GREEN,
        ),

        child: Align(
          alignment: Alignment.topCenter,
          child: SafeArea(
            left: true,
            top: true,
            right: true,
            bottom: true,
            child:SingleChildScrollView(child:


            Column(
              //mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Container(
                  height: 10,
                  color: constants.LIGHT_GREEN,
                ),
              Container(
                  height: 50,
                  decoration: BoxDecoration(
                    color: constants.LIGHT_GREEN,
                  ),
                  child:Row(
                    children: [
                      IconButton(
                        icon: Icon(Icons.arrow_back,color: Colors.white, size: 30,),
                        onPressed: ()=>onBackPressed(),
                      ),
                      Expanded(
                        child:  Container(
                          height: 20,
                          color: constants.LIGHT_GREEN,
                        ),
                      ),
                      IconButton(
                        icon: Icon(Icons.arrow_left,color: Colors.white, size: 30,),
                        onPressed: ()=>removeYearPress(),
                      ),
                      Text(
                        selectedYear.toString(),
                        style: TextStyle(
                          fontSize: 25,
                          color: constants.WHITE,
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                      IconButton(
                        icon: Icon(Icons.arrow_right,color: Colors.white, size: 30,),
                        onPressed: ()=>addYearPress(),
                      ),
                      SizedBox(width: 10,),
                    ],
                  )
              ),
                Container(
                  height: 40,
                  child: ListView.builder(
                    scrollDirection: Axis.horizontal,
                    itemCount: dateUtility.getMonths().length,
                    itemBuilder: (BuildContext context, int index) {
                      return GestureDetector(
                        onTap: ()=>onMonthPress(dateUtility.getMonths()[index]),
                        child: Container(
                            width: 100,
                            color: constants.LIGHT_GREEN,
                            child: Column(
                              children: [
                                Text(dateUtility.getMonths()[index],style: getMonthHighlightStyle(dateUtility.getMonths()[index]),),
                                SizedBox(height: 3,),
                                Container(
                                  decoration: BoxDecoration(
                                      color: getMonthHighlightColor(dateUtility.getMonths()[index]),
                                      borderRadius: BorderRadius.circular(1)
                                  ),
                                  width: 30,
                                  height: 2,
                                )
                              ],
                            )
                        ),
                      );
                    },
                  ),
                ),
                Container(
                  height: 65,
                  child: ListView.builder(
                    scrollDirection: Axis.horizontal,
                    itemCount: datesToShow.length,
                    itemBuilder: (BuildContext context, int index) {
                      return GestureDetector(
                        onTap: ()=>onDayPress(datesToShow[index]),
                          child: Padding(
                            padding: EdgeInsets.only(right: 7,left: 7),
                            child: Container(
                                width: 35,
                                decoration: BoxDecoration(
                                    color: getDayHighlightColor(datesToShow[index]),
                                    //color: Colors.cyan[300],
                                    borderRadius: BorderRadius.circular(30)
                                ),
                                child:Padding(
                                    padding: EdgeInsets.only(top: 7),
                                    child:Column(
                                      children: [
                                        Text(dateUtility.getDayName(datesToShow[index]),style: TextStyle(color: constants.FADING_WHITE, fontSize: 20),),
                                        Text(datesToShow[index].day.toString(),style: TextStyle(color: Colors.white, fontSize: 20),),
                                      ],
                                    )
                                )
                            ),
                          )
                      );
                    },
                    //separatorBuilder: (BuildContext context, int index) => const Divider(),
                  ),
                ),
                SizedBox(height: 20,),
                /*Expanded(
                    child:Stack(
                      children: [
                        Positioned.fill(*/
                             Container(
                                 constraints: BoxConstraints(
                                     minHeight: getHeight(), minWidth: double.infinity),
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(30),
                                color: constants.WHITE,
                                boxShadow: [
                                  BoxShadow(
                                    color: Colors.black.withOpacity(0.3),
                                    spreadRadius: 5,
                                    blurRadius: 7,
                                    offset: Offset(0, 3), // changes position of shadow
                                  ),
                                ],
                              ),
                              child:Column(
                                //mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                children: [
                                  SizedBox(height: 5,),
                                  Text(
                                      "Orario",
                                    style: TextStyle(
                                      fontSize: 20,
                                      color: constants.GREY,
                                      fontWeight: FontWeight.bold,
                                    ),
                                  ),
                                  Row(
                                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                    children: [
                                      Container(
                                          height: 100,
                                          width: 60,
                                          child:ListWheelScrollView.useDelegate(
                                            onSelectedItemChanged: (index) => setStartIndex(index+24),
                                            squeeze: 1.2,
                                            itemExtent: 30,
                                            diameterRatio: 1,
                                            childDelegate: ListWheelChildBuilderDelegate(
                                                builder: (BuildContext context, int index) {
                                                  index+=24;
                                                  if (index < 0 || index >= dateUtility.orari.length) {
                                                    return null;
                                                  }
                                                  return Text(
                                                    dateUtility.orari[index],
                                                    style: TextStyle(
                                                      fontSize: 20,
                                                      color: getColorStartHour(index),
                                                      fontWeight: FontWeight.bold,
                                                    ),
                                                  );
                                                }
                                            ),
                                          )
                                      ),
                                      Icon(Icons.arrow_right_alt, color: constants.GREY,size: 30,),
                                      Container(
                                          height: 100,
                                          width: 60,
                                          child:ListWheelScrollView.useDelegate(
                                            onSelectedItemChanged: (index) => setEndIndex(index+24),
                                            squeeze: 1.2,
                                            itemExtent: 30,
                                            diameterRatio: 1,
                                            childDelegate: ListWheelChildBuilderDelegate(
                                                builder: (BuildContext context, int index) {
                                                  index+=24;
                                                  if (index < 0 || index >= dateUtility.orari.length) {
                                                    return null;
                                                  }
                                                  return Text(
                                                    dateUtility.orari[index],
                                                    style: TextStyle(
                                                      fontSize: 20,
                                                      color: getColorEndHour(index),
                                                      fontWeight: FontWeight.bold,
                                                    ),
                                                  );
                                                }
                                            ),
                                          )
                                      )
                                    ],
                                  ),
                                  Text(
                                    "Titolo",
                                    style: TextStyle(
                                      fontSize: 20,
                                      color: constants.GREY,
                                      fontWeight: FontWeight.bold,
                                    ),
                                  ),
                                  Padding(
                                    padding: EdgeInsets.only(left: 20,right: 20),
                                    child: Container(
                                      height: 50,
                                      decoration: BoxDecoration(
                                        borderRadius: BorderRadius.circular(30),
                                        color: constants.DARK_GREEN,
                                        boxShadow: [
                                          BoxShadow(
                                            color: Colors.black.withOpacity(0.3),
                                            spreadRadius: 2,
                                            blurRadius: 7,
                                            offset: Offset(0, 3), // changes position of shadow
                                          ),
                                        ],
                                      ),
                                        child:Padding(
                                            padding: EdgeInsets.only(left:15,right: 15),
                                            child:TextField(
                                              controller: textDialogTitleController,
                                              decoration: InputDecoration(
                                                  border: InputBorder.none,
                                                  hintText: "Pulizia presso...",
                                                  hintStyle: TextStyle(fontSize: 20.0),
                                              ),
                                              style: TextStyle(fontSize: 20,),
                                              maxLines: 1,
                                            )
                                        )
                                    )
                                  ),
                                  SizedBox(height: 5,),
                                  Text(
                                    "Descrizione",
                                    style: TextStyle(
                                      fontSize: 20,
                                      color: constants.GREY,
                                      fontWeight: FontWeight.bold,
                                    ),
                                  ),
                                  Padding(
                                      padding: EdgeInsets.only(left:20,right: 20,),
                                      child: Container(
                                            decoration: BoxDecoration(
                                              borderRadius: BorderRadius.circular(30),
                                              color: constants.DARK_GREEN,
                                              boxShadow: [
                                                BoxShadow(
                                                  color: Colors.black.withOpacity(0.3),
                                                  spreadRadius: 2,
                                                  blurRadius: 7,
                                                  offset: Offset(0, 3), // changes position of shadow
                                                ),
                                              ],
                                            ),
                                          child:Padding(
                                              padding: EdgeInsets.only(left:15,right: 15,top:5,bottom: 5),
                                              child:TextField(
                                                controller: textDialogDescriptionController,
                                                decoration: InputDecoration(
                                                    border: InputBorder.none,
                                                    hintText: "L'incontro si svolgerà..."),
                                                keyboardType: TextInputType.multiline,
                                                minLines: 8,
                                                maxLines: null,
                                              )
                                          )
                                      )
                                    ),
                                  SizedBox(height: 10,),
                                  Padding(
                                    padding: EdgeInsets.only(left:30,right: 30,bottom: 10),
                                    child: Row(
                                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text(
                                          "N° minimo persone",
                                          style: TextStyle(
                                            fontSize: 20,
                                            color: constants.GREY,
                                            fontWeight: FontWeight.bold,
                                          ),
                                        ),
                                        Container(
                                            height: 50,
                                            width: 120,
                                            decoration: BoxDecoration(
                                              borderRadius: BorderRadius.circular(30),
                                              color: constants.DARK_GREEN,
                                              boxShadow: [
                                                BoxShadow(
                                                  color: Colors.black.withOpacity(0.3),
                                                  spreadRadius: 2,
                                                  blurRadius: 7,
                                                  offset: Offset(0, 3), // changes position of shadow
                                                ),
                                              ],
                                            ),
                                            child:Center(
                                                child:TextField(
                                                  controller: textDialogNumberController,
                                                  textAlign: TextAlign.center,
                                                  decoration: InputDecoration(
                                                    border: InputBorder.none,
                                                    hintText: 'Almeno 10',
                                                    hintStyle: TextStyle(fontSize: 20.0,),
                                                  ),
                                                  keyboardType: TextInputType.number ,
                                                  maxLines: 1,
                                                  style: TextStyle(
                                                    fontSize: 20,

                                                  ),

                                                )
                                            )
                                        )
                                      ],
                                    )
                                  ),
                                  Stack(
                                    children: [
                                      Container(
                                        color: constants.WHITE,
                                        height: 70,
                                        alignment: Alignment.center,
                                      ),
                                      Column(
                                        children: [
                                          Container(
                                            color: constants.WHITE,
                                            //height: getHeight()-453,
                                            constraints: BoxConstraints(
                                                minHeight: getMinBottomHeight(), minWidth: double.infinity),
                                            alignment: Alignment.center,
                                            child: GestureDetector(
                                              onTap: ()=>sendMeetingToServer(),
                                            child:Container(
                                              height: 50,
                                              width: 200,
                                              decoration: BoxDecoration(
                                                borderRadius: BorderRadius.circular(30),
                                                color: constants.DARK_GREEN,
                                                boxShadow: [
                                                  BoxShadow(
                                                    color: Colors.black.withOpacity(0.3),
                                                    spreadRadius: 2,
                                                    blurRadius: 7,
                                                    offset: Offset(0, 3), // changes position of shadow
                                                  ),
                                                ],
                                              ),
                                              child:Center(
                                                child:Text(
                                                "Conferma",
                                                style: TextStyle(
                                                  fontSize: 20,
                                                  color: constants.WHITE,
                                                  fontWeight: FontWeight.bold,
                                                ),
                                              ),
                                              ),
                                            ),
                                          ),
                                          ),
                                        ],
                                      ),




                                      /*
                                      Container(
                                              color: constants.WHITE,
                                              height: 60,
                                              alignment: Alignment.center,
                                              child: Container(
                                                height: 40,
                                                width: 200,
                                                decoration: BoxDecoration(
                                                  borderRadius: BorderRadius.circular(30),
                                                  color: constants.DARK_GREEN,
                                                ),
                                              )
                                          )
*/
                                    ],
                                  )
                                ],
                              )
                            ),

                        /*),
                        Positioned(
                          bottom: 0,
                          left:0,
                          right:0,
                          child: Container(
                            color: constants.WHITE,
                            height: 60,
                              alignment: Alignment.center,
                                child: Container(
                                  height: 40,
                                  width: 200,
                                  decoration: BoxDecoration(
                                    borderRadius: BorderRadius.circular(30),
                                    color: constants.DARK_GREEN,
                                  ),
                                )
                          )
                        )
                      ],
                    )
               */// )
            ],
            )
            )
          ),
        ),
      ),
    );
  }
}

class DateUtility{
  List<String> daysInEnglish=["Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"];
  List<String> daysInItalian=["Lunedì","Martedì","Mercoledì","Giovedì","Venerdì","Sabato","Domenica"];
  List<String> monthsInEnglish=["January","February","March","April","May","June","July","August","September","October","November","December"];
  List<String> monthsInItalian=["Gennaio","Febbraio","Marzo","Aprile","Maggio","Giugno","Luglio","Agosto","Settembre","Ottobre","Novembre","Dicembre"];
  List<String> orari=["00:00", "00:30", "01:00", "01:30", "02:00","02:30", "03:00", "03:30", "04:00", "04:30","05:00", "05:30", "06:00", "06:30", "07:00","07:30", "08:00", "08:30", "09:00", "09:30","10:00", "10:30", "11:00", "11:30", "12:00","12:30", "13:00", "13:30", "14:00", "14:30","15:00", "15:30", "16:00", "16:30", "17:00","17:30", "18:00", "18:30", "19:00", "19:30","20:00", "20:30", "21:00", "21:30", "22:00","22:30", "23:00", "23:30"];

  String getDayName(DateTime d){
    String day=DateFormat('EEEE').format(d).toString();
    for(int i=0;i<daysInEnglish.length;i++){
      if(day.compareTo(daysInEnglish[i])==0){
        return daysInItalian[i].substring(0,1);
      }
    }
  }

  List<Text> getOrariWidget(){
    List<Text> orariWidget=new List<Text>();
    for(int i=0;i<orari.length;i++){
      orariWidget.add(new Text(
        orari[i],
        style: TextStyle(
          fontSize: 20,
        ),
      ));
    }
    return getOrariWidget();
  }

  String getMonthName(DateTime d){
    String month=DateFormat('MMMM').format(d).toString();
    for(int i=0;i<daysInEnglish.length;i++){
      if(month.compareTo(monthsInEnglish[i])==0){
        return monthsInItalian[i];
      }
    }
  }


  List<DateTime> getDaysInBeteween(DateTime startDate, DateTime endDate) {
    List<DateTime> days = [];
    for (int i = 0; i <= endDate.difference(startDate).inDays; i++) {
      days.add(startDate.add(Duration(days: i)));
    }
    return days;
  }

  DateTime getLastDayInMonth(DateTime date){
    return DateTime.utc(date.year, date.month + 1, 0);
    //return new DateTime(date.year,date.month,getDaysInMonth(date));
  }

  int getDaysInMonth(DateTime date){
    var firstDayThisMonth = new DateTime(date.year, date.month, date.day);
    var firstDayNextMonth = new DateTime(firstDayThisMonth.year, firstDayThisMonth.month + 1, firstDayThisMonth.day);
    return firstDayNextMonth.difference(firstDayThisMonth).inDays;
  }

  DateTime getEndDate(DateTime startDate){
    return new DateTime(startDate.year+1, startDate.month, startDate.day);
  }

  List<String> getMonths(){
    return monthsInItalian;
  }

  bool isSameDate(DateTime dateTime1, DateTime dateTime2) {
    return dateTime1.year == dateTime2.year && dateTime1.month == dateTime2.month
        && dateTime1.day == dateTime2.day;
  }

  int getMonthNumber(String month){
    int pos=0;
    for(int i=0;i<monthsInItalian.length;i++){
      if(month.compareTo(monthsInItalian[i])==0){
        pos=i+1;
        return pos;
      }
    }
    return 0;
  }
}