import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'dart:async';
import 'package:flutter/material.dart';
import 'constants.dart' as constants;
import 'dart:ui';
import 'package:flutter/services.dart';
import 'package:geolocator/geolocator.dart';
import 'package:we_clean_project/loadingScreen.dart';
import 'package:tcp_socket_connection/tcp_socket_connection.dart';
import 'dart:convert';
import 'package:animated_icon_button/animated_icon_button.dart';
import 'package:we_clean_project/meetingMarker.dart';

class TestGoogleMap extends StatefulWidget {
  TestGoogleMap({Key key}) : super(key: key);

  @override
  _TestGoogleMap createState() => _TestGoogleMap();
}

class _TestGoogleMap extends State<TestGoogleMap> {
  Position position;
  TcpSocketConnection server=new TcpSocketConnection("192.168.1.110", constants.SERVER_PORT);
  double mapZoom=10;
  LatLng mapPosition=new LatLng(10,10);
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool isPinningLocation=false;
  List<MeetingMarker> meetingMarkers=new List<MeetingMarker>();
  LatLng pickedLocation;
  bool positionLoaded=false;
  Completer<GoogleMapController> _controllerMap = Completer();
  Set<Marker> markersToShow=new Set<Marker>();

  static final CameraPosition _kGooglePlex = CameraPosition(
    target: LatLng(37.42796133580664, -122.085749655962),
    zoom: 15,
  );

  static final CameraPosition _kLake = CameraPosition(
      bearing: 192.8334901395799,
      target: LatLng(37.43296265331129, -122.08832357078792),
      tilt: 59.440717697143555,
      zoom: 19.151926040649414);



  @override
  void initState() {
    setMarkerPosition();
    super.initState();
  }

  void setMarkerPosition() async{
    await _determinePosition();
    if(position==null){
      print("Position is null");
      return;
    }
    setState(() {
      markersToShow.add(new Marker(
        markerId: new MarkerId("PositionMarker"),
        position: new LatLng(position.latitude,position.longitude),

      ));
    });

    _goToPosition(new LatLng(position.latitude,position.longitude),15);
    print("Set Position Marker");
    askMeetingMarkers();
  }

  Future<void> _goToPosition(LatLng pos, double zoom) async {
    final GoogleMapController controller = await _controllerMap.future;
    controller.animateCamera(CameraUpdate.newCameraPosition(new CameraPosition(
      target: pos,
      zoom: zoom,
    )));
  }

  /////////TO FIND POINTS AT GIVEN DISTANCE YOU HAVE TO SEND LATITUDE AND LONGITUDE AND GIVEN DISTANCE. THEN THE SERVER WILL FIND THE DIFFERENCE BETWEEN LATITUDE AND LONGITUDE OF POINTS AND MY POSITION.
  /////////THEN I HAVE TO CHECK THAT            -SQUARED_ROOT((GIVEN_DISTANCE/111)-LATITUDE_DISTANCE^2))<=LONGITUDE_DISTANCE<=SQUARED_ROOT((GIVEN_DISTANCE/111)-LATITUDE_DISTANCE^2)) AND IF IT WORKS I HAVE
  /////////TO SEND BACK THAT POINT WITH ITS INFORMATION.

  void askMeetingMarkers() async{
    await server.connect(5000, "EOS", messageReceived);
    server.sendMessage("GETMEETINGBYPOSITION%_%"+position.latitude.toString()+"%_%"+position.longitude.toString()+"%_%EOS");
  }

  void messageReceived(String message){
    if(message.startsWith("NONE")){
      print("THERE ARE NO POINTS NEAR YOU");
    }
    if(message.startsWith("MEETINGS")){
      print("RECEIVED");
      message=message.replaceAll("MEETINGS%_%", "");
      List<MeetingMarker> markers;
      markers=(json.decode(message) as List).map((i) => MeetingMarker.fromJson(i)).toList();

      for(int i=0;i<markers.length;i++){
        if(markers[i].deadLine.compareTo("2000-01-01 00:00:00")==0){
          markers[i].deadLine=null;
        }
        markers[i].marker=new Marker(
          markerId: new MarkerId(markers[i].id),
          onTap: ()=>showMarkerInfo(markers[i]),
          position: new LatLng(markers[i].latitude,markers[i].longitude),
        );
        setState(() {
          markersToShow.add(markers[i].marker);
        });
        meetingMarkers.addAll(markers);
      }
    }
    server.disconnect();
  }

  void showMarkerInfo(MeetingMarker m){
    showDialog(
        context: context,
        builder: (context) {
          return StatefulBuilder(
              builder: (context, setState) {
                return Dialog(
                  backgroundColor: Colors.transparent,
                  child: new BackdropFilter(
                    filter: ImageFilter.blur(sigmaX: 5, sigmaY: 5),
                    child:SingleChildScrollView(

                      child: Container(
                          height: 470.0,
                          width: 400.0,
                          decoration: BoxDecoration(
                              color: constants.LIGHT_GREEN,
                              //color: Colors.cyan[300],
                              borderRadius: BorderRadius.circular(30)
                          ),
                          child: Padding(
                            padding: EdgeInsets.all(5),
                            child: Column(
                                children: <Widget>[
                                  Text(m.title),
                                  Text(m.startDateTime),
                                  Text(m.description),
                                  Text(m.maxPartecipants.toString()),
                                  //Text(m.)
                                ]
                            ),
                          )
                      ),
                    ),
                  ),
                );
              }
          );}
    ).then((val){
      print("CLOSED");
    });
  }

  void addMarker(MeetingMarker mark){
    server.connect(5000, "EOS", messageReceived);
    server.sendMessage("ADD_MARKER%_%"+mark.toJson().toString()+"%_%EOF");
  }


  _determinePosition() async {
    bool serviceEnabled;
    LocationPermission permission;

    serviceEnabled = await Geolocator.isLocationServiceEnabled();
    if (!serviceEnabled) {
      return Future.error('Location services are disabled.');
    }

    permission = await Geolocator.checkPermission();
    if (permission == LocationPermission.deniedForever) {
      return Future.error(
          'Location permissions are permantly denied, we cannot request permissions.');
    }

    if (permission == LocationPermission.denied) {
      permission = await Geolocator.requestPermission();
      if (permission != LocationPermission.whileInUse &&
          permission != LocationPermission.always) {
        return Future.error(
            'Location permissions are denied (actual value: $permission).');
      }
    }
    position= await Geolocator.getCurrentPosition();
    startTimeout(1500);

  }

  startTimeout([int milliseconds]) {
    var duration = milliseconds == null ? constants.timeout : constants.ms * milliseconds;
    return new Timer(duration, handleTimeout);
  }

  void handleTimeout() {  // callback function
    setState(() {
      positionLoaded=true;
    });
  }


  void tapMap(LatLng latlng){
    if(isPinningLocation) {
      removeAddedMarker();
      print(latlng);
      pickedLocation=latlng;
      setState(() {
        markersToShow.add(new Marker(
          markerId: new MarkerId("ChoosePositionMarker"),
          position: latlng,
        ));
      });
    }
  }

  void showMenu(){
    Navigator.push(
        context,
        MaterialPageRoute(builder: (context) => TestGoogleMap(
        )));
    /*
    Navigator.push(
        context,
        MaterialPageRoute(builder: (context) => MeetingSettings(
          position: pickedLocation,
        )));*/
  }

  void removeAddedMarker(){
    for(int i=1;i<markersToShow.length;i++){
      for(int k=0;k<meetingMarkers.length;k++){
        if(markersToShow.elementAt(i).markerId.value.compareTo("ChoosePositionMarker")==0){
          setState(() {
            markersToShow.remove(markersToShow.elementAt(i));
          });
          break;
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {



    GoogleMap googleMap=new GoogleMap(
      onTap: tapMap,
      mapType: MapType.normal,
      markers: markersToShow,
      zoomControlsEnabled: false,
      initialCameraPosition: _kGooglePlex,
      onMapCreated: (GoogleMapController controller) {
        _controllerMap.complete(controller);
      },
    );

    /*
    return new Scaffold(
      body: googleMap,
    );*/
    ListView menu=new ListView(
      children: [
        Container(
          height: 200,
          decoration: BoxDecoration(
            color: constants.LIGHT_GREEN,
          ),
        ),
        ButtonTheme(
          minWidth: double.infinity,
          child: MaterialButton(
            onPressed: () {print("BUTTON PRESSED");},
            child:Card(
              child: ListTile(
                leading: FlutterLogo(),
                title: Text('Hello'),
              ),
            ),
          ),
        ),
        ButtonTheme(
          minWidth: double.infinity,
          child: MaterialButton(
            onPressed: () {print("BUTTON PRESSED");},
            child:Card(
              child: ListTile(
                leading: FlutterLogo(),
                title: Text('Hello'),
              ),
            ),
          ),
        ),
        ButtonTheme(
          minWidth: double.infinity,
          child: MaterialButton(
            onPressed: () {print("BUTTON PRESSED");},
            child:Card(
              child: ListTile(
                leading: FlutterLogo(),
                title: Text('Hello'),
              ),
            ),
          ),
        ),
      ],
    );

    return  Scaffold(
        key:_scaffoldKey,
        //    **SIDE MENU**
        drawer: new Drawer(
          child: menu,
        ),
        body: AnnotatedRegion<SystemUiOverlayStyle>(
          value: SystemUiOverlayStyle(
            statusBarColor: constants.LIGHT_GREEN,
          ),

          child: Align(
            alignment: Alignment.topCenter,
            child: SafeArea(
              left: true,
              top: true,
              right: true,
              bottom: true,
              //minimum: const EdgeInsets.only(left:16.0, right: 16.0, bottom: 16.0),
              child:Stack(
                overflow: Overflow.visible,
                children: <Widget>[
                  Positioned(child: googleMap,),
                  Positioned(
                    child: Container(
                      height: 20,
                      color: constants.LIGHT_GREEN,
                    ),
                  ),
                  Positioned(
                    child: Container(
                      height: 60,
                      decoration: BoxDecoration(
                        color: constants.LIGHT_GREEN,
                        //color: Colors.blueGrey,
                        borderRadius: BorderRadius.circular(20),

                      ),
                      child:Row(
                        children: [
                          IconButton(
                            icon: Icon(Icons.menu,color: Colors.white, size: 30,),
                            onPressed: () {
                              _scaffoldKey.currentState.openDrawer();
                            },
                          ),
                          Expanded(
                            child:  Container(
                              height: 40,
                              child: Padding(
                                padding: EdgeInsets.only(left: 10,right:10),
                                child:Align(

                                  alignment: Alignment.centerLeft,
                                  child:Text(
                                    "Cerca...",
                                    style: TextStyle(
                                      color: Colors.white,
                                      fontSize: 20,
                                    ),
                                  ),
                                ),
                              ),
                              decoration: BoxDecoration(
                                color: constants.DARK_GREEN,
                                //color: Colors.blueGrey,
                                borderRadius: BorderRadius.circular(20),

                              ),
                            ),
                          ),

                          IconButton(
                            icon: Icon(Icons.search,color: Colors.white,size: 30,),
                            onPressed: () {
                              //_scaffoldKey.currentState.openDrawer();
                            },
                          ),
                        ],
                      ),


                    ),
                  ),
                  /*Positioned(
                    top: 70,
                      child: Container(
                      color: constants.LIGHT_GREEN,
                        child: Text(
                          "PREMI SULLA MAPPA PER SCEGLIERE UN PUNTO DI INCONTRO",
                          style: TextStyle(color: Colors.white,fontSize: 20),
                        ),
                    ),
                  ),*/
                  Positioned.fill(
                      child: Visibility(
                        visible: !positionLoaded,
                        child: LoadingScreen(),
                      )

                  ),
                ],

              ),
            ),
          ),
        ),

        floatingActionButton:  Stack(
          children: [
            Positioned(
              bottom:10,
              right:35,
              child:
              Visibility(
                child:Container(
                    width: 140.0,
                    height: 40.0,
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(15),
                      color:constants.LIGHT_GREEN,
                    ),
                    child: new RawMaterialButton(
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.all(Radius.circular(15.0)),
                      ),
                      elevation: 0.0,
                      child:Text("Conferma",style: TextStyle(color: Colors.white,fontSize: 18),),
                      onPressed: () =>showMenu(),
                    )
                ),
                visible: (pickedLocation!=null),
              ),
            ),

            Align(
              alignment: Alignment.bottomRight,
              child: Visibility(
                visible: positionLoaded,
                child:AnimatedIconButton(
                  endBackgroundColor: constants.LIGHT_GREEN,
                  startBackgroundColor: constants.LIGHT_GREEN,
                  size: 30,
                  onPressed: () {
                    if(isPinningLocation){
                      pickedLocation=null;
                      removeAddedMarker();
                    }else{

                    }
                    isPinningLocation=(!isPinningLocation);
                  },
                  duration: Duration(milliseconds: 300),
                  endIcon: Icon(
                    Icons.close,
                    color: constants.WHITE,
                  ),
                  startIcon: Icon(
                    Icons.add,
                    color: constants.WHITE,
                  ),
                ),
              ),
            ),
          ],
        )
    );
  }
/*
  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    throw UnimplementedError();
  }*/
/*
  Future<void> _goToTheLake() async {
    final GoogleMapController controller = await _controller.future;
    controller.animateCamera(CameraUpdate.newCameraPosition(_kLake));
  }*/
}
/*
class MeetingMarker{
  double latitude;
  double longitude;
  String title;
  int maxPartecipants;
  int actualPartecipants;
  String startDateTime;
  String endDateTime;
  String id;
  String description;
  Marker marker;
  String organizer;
  String deadLine="2000-01-01 00:00:00";

  MeetingMarker();

  MeetingMarker.fromJson(Map<String, dynamic> json)
      : latitude = json['latitude'],
        longitude = json['longitude'],
        title = json['title'],
        maxPartecipants = json['maxPartecipants'],
        actualPartecipants = json['actualPartecipants'],
        startDateTime = json['startDateTime'],
        endDateTime = json['endDateTime'],
        id=json['id'],
        description=json['description'],
        organizer=json['organizer'],
        deadLine=json['deadLine']
  ;

  Map<String, dynamic> toJson() =>
      {
        'latitude': latitude,
        'longitude': longitude,
        'title': title,
        'maxPartecipants': maxPartecipants,
        'actualPartecipants': actualPartecipants,
        'startDateTime': startDateTime,
        'endDateTime': endDateTime,
        'id': id,
        'description':description,
        'organizer':organizer,
        'deadLine':deadLine,
      };

  @override
  String toString() {
    return this.toJson().toString();
  }
}*/